/***************************************************************
 * Name:      wxextprocess.h
 * Purpose:
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2009-05-20
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   wxWindows licence
 **************************************************************/

#ifndef WXEXTPROCESS_H
#define WXEXTPROCESS_H

// Include additional wxWidgets headers.
#include <wx/process.h>


//////////////////////////////////////////////////////////
// Class wxExtProcess
//
/// \brief ...
///
/// ...
class wxExtProcess : public wxProcess
{
  public:
    /// \brief Standard constructor.
    ///
    /// \param poParent     ...
    /// \param nId=         ...
    wxExtProcess( wxEvtHandler * poParent, int nId )
      : wxProcess( poParent, nId ), m_nProcessId( 0 )
    {
    }

    /// Virtual destructor.
    virtual ~wxExtProcess()
    {
    }

    /// \brief ...
    ///
    /// \param strCommand   ...
    /// \param nSync=       ...
    virtual long Execute( const wxString& strCommand, int nSync= wxEXEC_ASYNC )
    {
        m_nProcessId= ::wxExecute( strCommand, nSync, this );
        return m_nProcessId;
    }

    /// \brief ...
    ///
    /// \param nProcessId   ...
    virtual void SetProcessId( long nProcessId )
    {
        m_nProcessId= nProcessId;
    }

    /// ...
    virtual long GetProcessId()
    {
        return m_nProcessId;
    }

    /// \brief ...
    ///
    /// \param nSignal      ...
    /// \param nFlags       ...
    virtual wxKillError Kill( wxSignal nSignal= wxSIGNONE, int nFlags= wxKILL_NOCHILDREN )
    {
        return wxProcess::Kill( GetProcessId(), nSignal, nFlags );
    }

  protected:
    long m_nProcessId;              ///< Holds the process id of the running sub process.
};


//////////////////////////////////////////////////////////
// Class wxExtProcessDelTempFile
//
/// \brief ...
///
/// ...
class wxExtProcessDelTempFile : public wxExtProcess
{
  public:
    /// Standard constructor.
    wxExtProcessDelTempFile( wxEvtHandler *poParent= NULL, int nId = -1, const wxString &strTempFileName= wxEmptyString )
      : wxExtProcess( poParent, nId )
    {
        m_strTempFileName= strTempFileName;
    }

    /// Virtual destructor.
    virtual ~wxExtProcessDelTempFile()
    {
        if( !m_strTempFileName.IsEmpty() && ::wxFileExists( m_strTempFileName ) )
        {
            // Remove the temporary file.
            ::wxRemoveFile( m_strTempFileName );
        }
    }

  protected:
    wxString m_strTempFileName;     ///< Holds the name of the temporary file to be removed in the destructor.
};

#endif // WXEXTPROCESS_H
