--[[**********************************************************
    * Name:      wxiscan.lua
    * Purpose:   Demo for implementing Lua addons for wxIScan.
    * Author:    Daniel Nell (daniel.nell@nellresearch.de)
    * Created:   2009-11-13
    * Copyright: Daniel Nell (www.nellresearch.de)
    * License:   GPL
    **********************************************************
--]]


-------------------------------------------------------------
-- Redefine output function(s).
--
function print( ... )
    local args= {...}       -- Table for the function parameters.
    local text= ""          -- Accumulates the text for the output.

    -- Iterate on the function parameters and concatenate
    for index, value in ipairs( args ) do
        text= text .. tostring( value ) .. "\t"
    end

    -- Output via LogMessage (wxLuaScript::LogMessage).
    LogMessage( text )
end


-------------------------------------------------------------
-- Functions calling menu handlers on the C++ side.
--
function ScanImage()
    return SendMenuEvent( ID_FILESCANIMAGE )
end

function PrintImage()
    return SendMenuEvent( ID_FILEPRINT )
end

function DocumentReadText( bAsync )
    if bAsync == nil then
        bAsync= true
    end
    if bAsync then
        return SendMenuEvent( ID_DOCUMENTREADTEXT )
    else
        return SendMenuEvent( ID_DOCUMENTREADTEXTSYNC )
    end
end

function DocumentPreviousPage()
    return SendMenuEvent( ID_DOCUMENTPREVIOUSPAGE )
end

function DocumentNextPage()
    return SendMenuEvent( ID_DOCUMENTNEXTPAGE )
end

function CmdStop()
    return SendMenuEvent( ID_CMDSTOP )
end


-------------------------------------------------------------
-- Main chunks (referred by the config file)
--
function CarbonCopy()
    return ScanImage() and PrintImage()
end

function ScanAndRead()
    return ScanImage() and CmdStop() and DocumentReadText()
end

function TurnPagesAndRead()
    if CmdStop() and DocumentReadText( false ) then
        while DocumentNextPage() and CmdStop() and DocumentReadText( false ) do
        end
    end
end

function Ocr2Ooo( Language )
    -- Handle default parameter(s).
    if Language == nil then
        Language= "de"
    end

    -- Create a temporary file(name).
    local TempFileName= os.tmpname()

    -- Create temporary filename(s) from basename.
    local TempFileNameText= TempFileName .. ".rtf"

    -- OCR commands for various languages.
    local OcrCmd=
    {
        ["de"]= "cuneiform -l ger -f rtf -o " .. TempFileNameText .. " " .. TempFileName;
        ["en"]= "cuneiform -l eng -f rtf -o " .. TempFileNameText .. " " .. TempFileName
    }

    -- Convert the image to a monochrome image.
    Image2Mono()

    -- Save the image to the given name (as tiff file).
    SaveImage( TempFileName, "image/tiff" )

    -- Execute the OCR software and create a rich text file.
    Execute( OcrCmd[Language] )

    -- Open openoffice.org for editing.
    os.execute( "soffice " .. TempFileNameText )

    -- Remove temporary files.
    os.remove( TempFileName )
    os.remove( TempFileNameText )
end

function ReadOut( Language )
    -- Handle default parameter(s).
    if Language == nil then
        Language= "de"
    end

    -- Create a temporary file(name).
    local TempFileName= os.tmpname()

    -- Create temporary filename(s) from basename.
    local TempFileNameText= TempFileName .. ".txt"

    -- OCR commands for various languages.
    local OcrCmd=
    {
        ["de"]= "cuneiform -l ger -f text -o " .. TempFileNameText .. " " .. TempFileName;
        ["en"]= "cuneiform -l eng -f text -o " .. TempFileNameText .. " " .. TempFileName
    }

    -- Read out commands for various languages.
    local ReadOutCmd=
    {
        ["de"]= "sh -c 'espeak -p 130 -v mb-de6 -f " .. TempFileNameText
                .. " | mbrola -e /usr/share/mbrola/voices/de6 - - | aplay -r22050 -fS16'";
        ["en"]= "sh -c 'espeak -s 110 -p 10 -v mb-en1 -f " .. TempFileNameText
                .. " | mbrola -e /usr/share/mbrola/voices/en1 - - | aplay -r22050 -fS16' "
    }

    -- Convert the image to a monochrome image.
    Image2Mono()

    -- Save the image to the given name (as TIFF file).
    SaveImage( TempFileName, "image/tiff" )

    -- Execute the OCR software and create a plain text file.
    Execute( OcrCmd[Language] )

    -- Read out the text file.
    Execute( ReadOutCmd[Language], true, TempFileNameText )

    -- Remove temporary files.
    os.remove( TempFileName )
end

function Test()
    print( "LUA: " .. _VERSION )
    print( "LUA: Current date and time is " .. os.date( "%x %X", os.time() ) .. '.' )
    print( "LUA: Your home directory is '" .. os.getenv( "HOME" ) .. "'." )
    EnhenceColours( -50, 100 )
    Image2Mono()
    EnhenceOpticalContrast()
end
