/***************************************************************
 * Name:      wxluascript.h
 * Purpose:   Defines a lua scripting object
 * Author:    Daniel Nell (daniel.nell@nellresearch.de)
 * Created:   2009-11-11
 * Copyright: Daniel Nell (www.nellresearch.de)
 * License:   wxWindows license
 **************************************************************/

// Include precompiled headers.
#include "wx_pch.h"

// wxWidgets headers
// ...

// Private header files.
#include "wxluascript.h"


/////////////////////////////////////////////////////////////////////////////
// Class wxLuaScript (scripting support)
//
// ...
//

#if __WXLUASCRIPT_DYNAMIC__
// Define dynamic library.
wxDynamicLibrary wxLuaScript::m_oWxLuaScriptDynamicLibrary;

// Define and initialize dynamic lua functions with NULL;
wxLuaScript::t_luaL_newstate *wxLuaScript::luaL_newstate= NULL;
wxLuaScript::t_luaL_openlibs *wxLuaScript::luaL_openlibs= NULL;
wxLuaScript::t_lua_atpanic *wxLuaScript::lua_atpanic= NULL;
wxLuaScript::t_lua_error *wxLuaScript::lua_error= NULL;
wxLuaScript::t_lua_gettop *wxLuaScript::lua_gettop= NULL;
wxLuaScript::t_luaL_loadstring *wxLuaScript::luaL_loadstring= NULL;
wxLuaScript::t_lua_pushstring *wxLuaScript::lua_pushstring= NULL;
wxLuaScript::t_lua_pushinteger *wxLuaScript::lua_pushinteger= NULL;
wxLuaScript::t_lua_pushboolean *wxLuaScript::lua_pushboolean= NULL;
wxLuaScript::t_lua_tointeger *wxLuaScript::lua_tointeger= NULL;
wxLuaScript::t_lua_tolstring *wxLuaScript::lua_tolstring= NULL;
wxLuaScript::t_lua_tonumber *wxLuaScript::lua_tonumber= NULL;
wxLuaScript::t_lua_toboolean *wxLuaScript::lua_toboolean= NULL;
wxLuaScript::t_lua_isnumber *wxLuaScript::lua_isnumber= NULL;
wxLuaScript::t_lua_type *wxLuaScript::lua_type= NULL;
wxLuaScript::t_lua_isstring *wxLuaScript::lua_isstring= NULL;
wxLuaScript::t_lua_pushcclosure *wxLuaScript::lua_pushcclosure= NULL;
wxLuaScript::t_lua_pcall *wxLuaScript::lua_pcall= NULL;
wxLuaScript::t_lua_call *wxLuaScript::lua_call= NULL;
wxLuaScript::t_lua_settable *wxLuaScript::lua_settable= NULL;
wxLuaScript::t_lua_setfield *wxLuaScript::lua_setfield= NULL;
wxLuaScript::t_lua_getfield *wxLuaScript::lua_getfield= NULL;
wxLuaScript::t_lua_close *wxLuaScript::lua_close= NULL;
#endif // __WXLUASCRIPT_DYNAMIC__

// Standard constructor.
//
wxLuaScript::wxLuaScript( const wxString& strScript, wxEvtHandler *poParent, bool *pbParentState )
{
    // Initialize members.
    m_poParent= poParent;
    m_strScript= strScript;
    m_pbParentState= pbParentState;
    if( m_pbParentState )
    {
        (*m_pbParentState)= true;
    }

#if __WXLUASCRIPT_DYNAMIC__
    if( !m_oWxLuaScriptDynamicLibrary.IsLoaded() )
    {
        if( m_oWxLuaScriptDynamicLibrary.Load( wxDynamicLibrary::CanonicalizeName( wxT( WXLUASCRIPT_LIBLUA ) ) ) )
        {
            luaL_newstate= (t_luaL_newstate *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "luaL_newstate" ) );
            luaL_openlibs= (t_luaL_openlibs *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "luaL_openlibs" ) );
            lua_atpanic= (t_lua_atpanic *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_atpanic" ) );
            lua_error= (t_lua_error *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_error" ) );
            lua_gettop= (t_lua_gettop *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_gettop" ) );
            luaL_loadstring= (t_luaL_loadstring *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "luaL_loadstring" ) );
            lua_pushstring= (t_lua_pushstring *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_pushstring" ) );
            lua_pushinteger= (t_lua_pushinteger *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_pushinteger" ) );
            lua_pushboolean= (t_lua_pushboolean *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_pushboolean" ) );
            lua_tointeger= (t_lua_tointeger *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_tointeger" ) );
            lua_tolstring= (t_lua_tolstring *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_tolstring" ) );
            lua_tonumber= (t_lua_tonumber *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_tonumber" ) );
            lua_toboolean= (t_lua_toboolean *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_toboolean" ) );
            lua_isnumber= (t_lua_isnumber *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_isnumber" ) );
            lua_type= (t_lua_type *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_type" ) );
            lua_isstring= (t_lua_isstring *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_isstring" ) );
            lua_pushcclosure= (t_lua_pushcclosure *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_pushcclosure" ) );
            lua_call= (t_lua_call *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_call" ) );
            lua_pcall= (t_lua_pcall *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_pcall" ) );
            lua_settable= (t_lua_settable *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_settable" ) );
            lua_setfield= (t_lua_setfield *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_setfield" ) );
            lua_getfield= (t_lua_getfield *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_getfield" ) );
            lua_close= (t_lua_close *)m_oWxLuaScriptDynamicLibrary.GetSymbol( wxT( "lua_close" ) );
        }
        else
        {
            // Log error message.
            ::wxLogError( wxString( wxT( "wxLuaScript::wxLuaScript -- " ) )
                          + _( "Cannot load Lua library." ) );
            return;

        }
    }
#endif // __WXLUASCRIPT_DYNAMIC__

    // Initialize Lua.
    L= luaL_newstate();

    // Load Lua base libraries.
    luaL_openlibs( L );

    // Register an error handler (since there may be fatal Lua errors).
    lua_atpanic( L, LuaAtPanic );

    // Register the this pointer (to the current wxLuaScript object)
    // as a global value called "this".
    lua_pushinteger( L, (long)this );
    lua_setglobal( L, "this" );

    // Register "SendMenuEvent" as Lua function.
    RegisterFunction( wxT( "SendMenuEvent" ), SendMenuEvent );

    // Register "LogMessage" as Lua function.
    RegisterFunction( wxT( "LogMessage" ), LogMessage );
}

// Virtual destructor.
//
wxLuaScript::~wxLuaScript()
{
    // Cleanup Lua.
    lua_close( L );
}

// Get the this-pointer from Lua.
//
wxLuaScript *wxLuaScript::GetThis( lua_State* L )
{
    // Get the this-pointer from Lua.
    lua_getglobal( L, "this" );
    return (wxLuaScript *)lua_tointeger( L, -1 );
}

// Error handler for (maybe fatal) Lua errors.
//
int wxLuaScript::LuaAtPanic( lua_State* L )
{
    // Get the error message and log it.
    ::wxLogError( wxString( (const char *)lua_tostring( L, 1 ), wxConvISO8859_1, wxSTRING_MAXLEN ) );

    // Return the number of (Lua) results.
	return 0;
}

// Log a message on the standard wxWidgets log target.
//
// Note: You can change the standard log target and the
//       messages you log are logged to the new target.
//
int wxLuaScript::LogMessage( lua_State* L )
{
    // Check the number of argument(s) and the type(s).
    if( ( lua_gettop( L ) != 1 ) || !lua_isstring( L, 1 ) )
    {
        // Signal error.
        lua_pushstring( L, "Incorrect argument or incorrect number of arguments." );
        lua_error( L );
    }

    // Get the message and log it.
    ::wxLogMessage( wxString( (const char *)lua_tostring( L, 1 ), wxConvISO8859_1, wxSTRING_MAXLEN ) );

    // Return the number of (Lua) results.
	return 0;
}

// Send a menu event to the parent event handler.
//
int wxLuaScript::SendMenuEvent( lua_State* L )
{
    // Check the number of (Lua) argument(s) and the type(s).
    if( ( lua_gettop( L ) != 1 ) || !lua_isnumber( L, 1 ) )
    {
        // Signal error.
        lua_pushstring( L, "Incorrect argument or incorrect number of arguments." );
        lua_error( L );
    }

    // Get the this pointer to the current wxLuaScript object.
    wxLuaScript *poThis= GetThis( L );

    // Get the pointer to the event handler.
    wxEvtHandler *poParent= poThis->m_poParent;

    // Post the message (if possible).
    if( poParent )
    {
        // Process the corresponding wxWidgets menu event.
        //
        // NOTE: We get the adress of the this-pointer from Lua
        //       (for this is a static function, and we need it
        //       to call the correct event handler).
        wxCommandEvent oEvent( wxEVT_COMMAND_MENU_SELECTED, lua_tonumber( L, 1 ) );
        poParent->ProcessEvent( oEvent );

        // Push the return value ( the parent state );
        if( poThis->m_pbParentState )
        {
            lua_pushboolean( L, *poThis->m_pbParentState );
            (*poThis->m_pbParentState)= true;
        }
        else
        {
            lua_pushboolean( L, true );
        }
    }
    else
    {
        // Push an error value for there is no parent so
        // post the message.
        lua_pushboolean( L, false );
    }

    // Return the number of (Lua) results.
	return 1;
}
